//
//  HA7 logger mini_AppDelegate.m
//  HA7 logger mini
//  Copyright(C)2012 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		Simple vhecle data logger for "HONDA GBD-HA7".
//		Needs the special USB device.
//		That is a FTDI FT232R(USB UART converter) and ISO9141-2 K-Line interface circuit.
//
//	Usage
//		Install FTDI D2XX driver software.
//		Connect a PC to USB adapter.
//		Connect a USB adapter to vhecle's OBD2 socket.
//		Start up this program.
//
//	Environment:
//		Runtime:
//			Mac OS X 10.4 -
//		Development:
//			Mac OS X 10.5.8
//			Xcode 3.1.2
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2012.01.02 v0.0  First cording
//    2012.01.12 v0.1  Commit
//

#import "HA7 logger mini_AppDelegate.h"

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "iso9141_io.h"

static FILE	*fp = NULL;

@implementation HA7_logger_mini_AppDelegate

@synthesize window;

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
	// Insert code here to initialize your application 
}


- (IBAction)StartButton:(id)sender
{
	int				rc;
	char			dev[64];
	unsigned char	keyword[2];
	
	if (fp == NULL) {
		char fname[256];
		
		sprintf(fname, "%s/Desktop/HA7.csv", getenv("HOME"));
		if ((fp = fopen(fname, "a")) == NULL) {
			[msg setStringValue:@"Log file open error"];
			return;
		}
		fprintf(fp, "DateTime, ect[˚C], shrtft1[%%], longft1[%%], map[kPa], rpm[rpm], vss[km/h], sparkadv[˚], iat[˚C], tp[%%], o2s11[V], shrtft11[%%]\n");
	}
	
	if (updateTimer == nil) {		
		//
		// Connect to USB -> FT232R
		//
		
		//		[msg setStringValue:@"Connect OBD2/ISO9141-2 via USB UART FT232R"];
		if ((rc = iso9141_open(dev)) != 0) {
			[msg setStringValue:@"Not found FTDI FT232R USB device"];
			return;
		}
		
		//
		// Connect to FT232R -> OBD2 -> ISO-9141-2
		//
		
		//		[msg setStringValue:@"Connect to vehicle..."];
		if ((rc = iso9141_init(keyword)) != 0) {
			[msg setStringValue:@"Not response a vehicle"];
			return;
		}
		if ((keyword[0] == 0x08) && (keyword[1] == 0x08)) {
			[msg setStringValue:@"Connected: Protocol is ISO-9141-2"];
		} else if ((keyword[0] == 0x94) && (keyword[1] == 0x94)) {
			[msg setStringValue:@"Connected: Protocol is ISO-9141-2(0x9494)"];
		} else {
			[msg setStringValue:@"Error: Unknown protocol"];
			return;
		}
		
		//
		// Start collection
		//
		updateTimer = [NSTimer scheduledTimerWithTimeInterval:2
													   target:self
													 selector:@selector(GetData:)
													 userInfo:nil
													  repeats:YES];
	}
}


- (IBAction)GetData:(id)sender
{
	int				rc;
	size_t			len;
	unsigned char	buf[16];
	int				i;
	
	//
	// Scan values
	//
	for(;;) {
		int	pid[] = {0x05, 0x06, 0x07, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x11, 0x14};
		int		ect;
		float	shrtft1;
		float	longft1;
		int		map;
		int		rpm;
		int		vss;
		int		sparkadv;
		int		iat;
		float	tp;
		float	o2s11;
		float	shrtft11;
		
		for(i = 0; i < sizeof(pid); i++) {
			
			buf[0] = 0x01;	// Request current powertrain diagnostic data request SID
			buf[1] = pid[i];
			if ((rc = iso9141_send_msg(buf, 2)) != 0) {
				break;
			}
			if ((rc = iso9141_recv_response(buf, &len)) != 0) {
				break;
			}
			
			switch(i){
				case 0:
					ect = buf[2] - 40;
					break;
				case 1:
					shrtft1 = (buf[2] - 128) * 100 / 128.0;
					break;
				case 2:
					longft1 = (buf[2] - 128) * 100 / 128.0;
					break;
				case 3:
					map = buf[2];
					break;
				case 4:
					rpm = (buf[2] *256 + buf[3]) / 4;
					break;
				case 5:
					vss = buf[2];
					break;
				case 6:
					sparkadv = (buf[2] - 128) / 2;
					break;
				case 7:
					iat = buf[2] - 40;
					break;
				case 8:
					tp = buf[2] * 100 / 255.0;
					break;
				case 9:
					o2s11 = buf[2] * 0.005;
					shrtft11 = (buf[3] - 128) * 100 / 128.0;
					break;
				default:
					break;
			}
			
		}
		
		[ect_v setIntValue:ect];
		[ect_g setIntValue:ect];
		
		[shrtft1_v setFloatValue:shrtft1];
		[shrtft1_g setFloatValue:shrtft1];
		
		[longft1_v setFloatValue:longft1];
		[longft1_g setFloatValue:longft1];
		
		[map_v setIntValue:map];
		[map_g setIntValue:map];
		
		[rpm_v setIntValue:rpm];
		[rpm_g setIntValue:rpm];
		
		[vss_v setIntValue:vss];
		[vss_g setIntValue:vss];
		
		[sparkadv_v setIntValue:sparkadv];
		[sparkadv_g setIntValue:sparkadv];
		
		[iat_v setIntValue:iat];
		[iat_g setIntValue:iat];
		
		[tp_v setFloatValue:tp];
		[tp_g setFloatValue:tp];
		
		[o2s11_v setFloatValue:o2s11];
		[o2s11_g setFloatValue:o2s11 * 1000.0];
		
		[shrtft11_v setFloatValue:shrtft11];
		[shrtft11_g setFloatValue:shrtft11];
		
		if (fp != NULL) {
			time_t	t;
			struct tm *tm;
			
			time(&t);
			tm = localtime(&t);
			fprintf(fp, "%d-%02d-%02d %02d:%02d:%02d, ",
					tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday, tm->tm_hour, tm->tm_min, tm->tm_sec);
			fprintf(fp, "%d, %f, %f, %d, %d, %d, %d, %d, %f, %f, %f\n",
					ect, shrtft1, longft1, map, rpm, vss, sparkadv, iat, tp, o2s11, shrtft11);
		}
		
		break;
	}
	
}


- (IBAction)StopButton:(id)sender
{	
	if (updateTimer != nil) {
		[updateTimer invalidate];
		updateTimer = nil;
		[msg setStringValue:@"Disconnected"];
	}
	
	iso9141_close();	
	
	if (fp != NULL) {
		fclose(fp);
		fp = NULL;
	}
	
}

- (void)windowWillClose:(NSNotification *)aNotification
{
	[self StopButton:nil];
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)theApplication
{
	return(YES);
}

@end
